/*
    Ming, an SWF output library
    Copyright (C) 2001  Opaque Industries - http://www.opaque.net/

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "blocks/_cl.h"

#include <stdlib.h>
#include <math.h>
#include "movie.h"
#include "shape_util.h"
#include "blocks/method.h"
#include "ming.h"

static void destroySWFExports(SWFMovie movie)
{
  int n;

  for(n=0; n<movie->nExports; ++n)
    _free_(movie->exports[n].name);

  _free_(movie->exports);

  movie->exports = 0;
  movie->nExports = 0;
}

WM_DLL_A void WM_DLL_B destroySWFMovie(SWFMovie movie)
{
  destroySWFBlockList(movie->blockList);
  destroySWFDisplayList(movie->displayList);
  destroySWFRect(movie->bounds);

  if(movie->nExports > 0)
    destroySWFExports(movie);

  _free_(movie);
}

extern int SWF_gNumCharacters;

WM_DLL_A SWFMovie WM_DLL_B  newSWFMovieWithVersion(int version)
{
  SWFMovie movie = _calloc_(1, SWFMOVIE_SIZE);

  movie->version = version;
  movie->blockList = newSWFBlockList();
  movie->displayList = newSWFDisplayList();
  movie->bounds = newSWFRect(0, 320*20, 0, 240*20);
  movie->rate = 12.0;
  movie->totalFrames = 1;

  movie->nExports = 0;
  movie->exports = NULL;

  SWFMovie_addBlock(movie, newSWFSetBackgroundBlock(0xff,0xff,0xff));

  return movie;
}

WM_DLL_A SWFMovie WM_DLL_B newSWFMovie()
{
  return newSWFMovieWithVersion(SWF_versionNum);
}

WM_DLL_A void WM_DLL_B SWFMovie_setRate(SWFMovie movie, float rate)
{
  movie->rate = rate;
}

extern float Ming_scale;

WM_DLL_A void WM_DLL_B SWFMovie_setDimension(SWFMovie movie, float width, float height)
{
  if(movie->bounds)
    _free_(movie->bounds);

  movie->bounds = newSWFRect(0, (int)rint(Ming_scale*width),
			     0, (int)rint(Ming_scale*height));
}

WM_DLL_A void WM_DLL_B SWFMovie_setNumberOfFrames(SWFMovie movie, int totalFrames)
{
  movie->totalFrames = totalFrames;
}

WM_DLL_A void WM_DLL_B SWFMovie_setBackground(SWFMovie movie, int r, int g, int b)
{
  destroySWFBlock(movie->blockList->blocks[0].block);
  movie->blockList->blocks[0].block = newSWFSetBackgroundBlock((byte)r,(byte)g,(byte)b);
}

WM_DLL_A void WM_DLL_B SWFMovie_addBlock(SWFMovie movie, SWFBlock block)
{
  if(SWFBlock_getType(block) == SWF_SHOWFRAME)
    ++(movie->nFrames);

  SWFBlockList_addBlock(movie->blockList, block);
}

WM_DLL_A void WM_DLL_B SWFMovie_addExport(SWFMovie movie, SWFBlock block, char *name)
{
  if(SWFBlock_getType(block) == SWF_DEFINESPRITE)
  {
    movie->exports = _realloc_(movie->exports,
			     (movie->nExports+1) * sizeof(struct swfexport));

    movie->exports[movie->nExports].block = block;
    movie->exports[movie->nExports].name = _strdup_(name);

    ++(movie->nExports);
  }
}

/* from displaylist.c */
WM_DLL_A void WM_DLL_B resolveDependencies(SWFCharacter character, SWFBlockList list);

WM_DLL_A void WM_DLL_B SWFMovie_writeExports(SWFMovie movie)
{
  int n;

  if(movie->nExports == 0)
    return;

  for(n=0 ; n<movie->nExports ; ++n)
  {
    SWFBlock b = movie->exports[n].block;

    if(!SWFBlock_isDefined(b))
    {
      resolveDependencies(b, movie->blockList);
      completeSWFBlock(b);
      SWFMovie_addBlock(movie, b);
    }
  }

  SWFMovie_addBlock(movie,
		    newSWFExportBlock(movie->exports, movie->nExports));

  destroySWFExports(movie);
}

static SWFDisplayItem *DisplayItemsAdded;
static char *DisplayItemsIgnored;
static int nDisplayItemsAdded;
static int nMaxDisplayItemsAdded;
#define MOVIE_ITEMS_ADDED_INCR 1024
static int ntotal_display_items_added;
static int ntest_compactify_display_items_added_at=1024;
static void CompactifyDisplayItemsArray(void)
{
 int i=0;
 int ipos=0;
 for(i=0;i<nDisplayItemsAdded;i++)
  if(DisplayItemsAdded[i])
  {
   if(ipos==i)
    ipos++;
   else
   {
    SWFDisplayItem item=DisplayItemsAdded[i];
    item->nrecord=i;
    DisplayItemsIgnored[ipos]=DisplayItemsIgnored[i];
    DisplayItemsAdded[ipos++]=DisplayItemsAdded[i];
   }
  }
 nDisplayItemsAdded=ipos;
 ntest_compactify_display_items_added_at=ntotal_display_items_added+max(1024,nDisplayItemsAdded/2);
}

WM_DLL_A void WM_DLL_B MingRecordDisplayItemAdded(SWFDisplayItem item)
{
 int i=0;
 /**/
 for(i=0;i<nDisplayItemsAdded;i++)
  if(DisplayItemsAdded[i]==NULL)
  {
   DisplayItemsAdded[i]=item;
   return;
  }
 if(nDisplayItemsAdded+1>nMaxDisplayItemsAdded)
 {
  int new_nMaxDisplayItemsAdded=nMaxDisplayItemsAdded+MOVIE_ITEMS_ADDED_INCR;
  char *NewDisplayItemsIgnored=(char *)_realloc_((void *)DisplayItemsIgnored,new_nMaxDisplayItemsAdded);
  SWFDisplayItem *NewDisplayItemsAdded=(SWFDisplayItem*)_realloc_((void *)DisplayItemsAdded,new_nMaxDisplayItemsAdded*sizeof(SWFDisplayItem *));
  if(NewDisplayItemsAdded&&NewDisplayItemsIgnored)
  {
   DisplayItemsAdded=NewDisplayItemsAdded;
   DisplayItemsIgnored=NewDisplayItemsIgnored;
   nMaxDisplayItemsAdded=new_nMaxDisplayItemsAdded;
  }
  else
  {
   if(NewDisplayItemsAdded)
    free(NewDisplayItemsAdded);
   if(NewDisplayItemsIgnored)
    free(NewDisplayItemsIgnored);
   return;
  }
 }
 DisplayItemsIgnored[nDisplayItemsAdded]=0;
 item->nrecord=nDisplayItemsAdded;
 DisplayItemsAdded[nDisplayItemsAdded++]=item;
 ntotal_display_items_added++;
 if(ntotal_display_items_added>=ntest_compactify_display_items_added_at)
  CompactifyDisplayItemsArray();
 return;
}


WM_DLL_A void WM_DLL_B MingForgetDisplayItem(SWFDisplayItem item)
{
 int i=item->nrecord;
 if(i>=0&&i<nDisplayItemsAdded)
  if(DisplayItemsAdded[i]==item)
 {
  DisplayItemsIgnored[i]=1;
  return;
 }
 return;
}

WM_DLL_A void WM_DLL_B MingRememberAllDisplayItems(SWFDisplayItem item)
{
 int i=0;
 for(i=0;i<nDisplayItemsAdded;i++)
  DisplayItemsIgnored[i]=0;
 return;
}



WM_DLL_A void WM_DLL_B MingUnRecordDisplayItemRemoved(SWFDisplayItem item)
{
 int i=item->nrecord;
 if(i>=0&&i<nDisplayItemsAdded)
  if(DisplayItemsAdded[i]==item)
 {
  DisplayItemsAdded[i]=NULL;
  DisplayItemsIgnored[i]=0;
  return;
 }
 return;
}

WM_DLL_A void WM_DLL_B MingRemoveAllDisplayItems(char remove_forgotten_items)
{
 int i=0;
 char found_forgotten_item=0;
 for(i=0;i<nDisplayItemsAdded;i++)
 {
  if(DisplayItemsAdded[i]&&DisplayItemsIgnored[i])
   found_forgotten_item=1;
  if(remove_forgotten_items||!DisplayItemsIgnored[i])
  if(DisplayItemsAdded[i])
  {
   SWFDisplayItem_remove(DisplayItemsAdded[i]);
   DisplayItemsAdded[i]=NULL;
  }
 }
 if(remove_forgotten_items||!found_forgotten_item)
 {
  if(DisplayItemsAdded)
  _free_(DisplayItemsAdded);
  if(DisplayItemsIgnored)
  _free_(DisplayItemsIgnored);
  DisplayItemsIgnored=NULL;
  DisplayItemsAdded=NULL;
  nMaxDisplayItemsAdded=nDisplayItemsAdded=0;
 }
 return;
}

WM_DLL_A SWFDisplayItem WM_DLL_B  SWFMovie_add(SWFMovie movie, SWFBlock block)
{
  if(block == NULL)
    return NULL;

  /* if they're trying to add a raw bitmap, we'll be nice and turn
     it into a shape */

  if(SWFBlock_getType(block) == SWF_DEFINEBITS ||
     SWFBlock_getType(block) == SWF_DEFINEBITSJPEG2 ||
     SWFBlock_getType(block) == SWF_DEFINEBITSJPEG3 ||
     SWFBlock_getType(block) == SWF_DEFINELOSSLESS ||
     SWFBlock_getType(block) == SWF_DEFINELOSSLESS2)
  {
    SWFShape shape = newSWFShape();
    SWFFill fill = SWFShape_addBitmapFill(shape, block, SWFFILL_TILED_BITMAP);

    float width = SWFCharacter_getWidth((SWFCharacter)block);
    float height = SWFCharacter_getHeight((SWFCharacter)block);

    SWFShape_setRightFill(shape, fill);

    /* bitmap's bounds aren't scaled, so drawCharacterBounds won't work */
    SWFShape_drawLine(shape, Ming_scale*width, 0);
    SWFShape_drawLine(shape, 0, Ming_scale*height);
    SWFShape_drawLine(shape, -Ming_scale*width, 0);
    SWFShape_drawLine(shape, 0, -Ming_scale*height);

    block = (SWFBlock)shape;
  }

  if(SWFBlock_isCharacter(block))
  {
    SWFDisplayItem item=SWFDisplayList_add(movie->displayList, (SWFCharacter)block);
    MingRecordDisplayItemAdded(item);
    return item;
  }
  else
    SWFMovie_addBlock(movie, block);

  return NULL;
}

WM_DLL_A void WM_DLL_B SWFMovie_remove(SWFMovie movie, SWFDisplayItem item)
{
  MingUnRecordDisplayItemRemoved(item);
  SWFDisplayItem_remove(item);
}

#ifdef cl_sound
WM_DLL_A void WM_DLL_B SWFMovie_setSoundStream(SWFMovie movie, SWFSound sound)
{
  SWFBlock block = SWFSound_getStreamHead(sound, movie->rate);

  if(block != NULL)
  {
    SWFMovie_addBlock(movie, block);
    SWFDisplayList_setSoundStream(movie->displayList, sound);
  }
}
#endif

WM_DLL_A void WM_DLL_B SWFMovie_nextFrame(SWFMovie movie)
{
  SWFDisplayList_writeBlocks(movie->displayList, movie->blockList);
  SWFMovie_addBlock(movie, newSWFShowFrameBlock());
}

WM_DLL_A void WM_DLL_B SWFMovie_labelFrame(SWFMovie movie, char *label)
{
  SWFMovie_addBlock(movie, newSWFFrameLabelBlock(label));
}


WM_DLL_A int WM_DLL_B SWFMovie_save(SWFMovie movie, char *filename)
{
  SFILE *sf = fopen_Buf(filename, "wb");
  int count;

  if(sf == NULL)
    return 0;

  count = SWFMovie_output(movie, fileOutputMethod, sf);
  fclose_Buf(sf);
  return count;
}


#ifdef _DEBUG_OUTPUT_FOR_FILE_POINTER
#define DEBUG_STRING(sz)\
  {\
   FILE *fp=fopen("debug.txt","a");\
   if(fp)\
   {\
    fprintf(fp,sz);\
    fclose(fp);\
   }\
  }
#else
#define DEBUG_STRING(sz)
#endif


WM_DLL_A int WM_DLL_B SWFMovie_output_for_file_name(SWFMovie movie, char *szOutputFile)
{
 SFILE *sf=fopen_Buf(szOutputFile,"wb");
 if(sf)
 {
  int iret=SWFMovie_output(movie, fileOutputMethod, sf);
  fclose_Buf(sf);
  return iret;
 }
 return 0;
}


WM_DLL_A int WM_DLL_B SWFMovie_output(SWFMovie movie, SWFByteOutputMethod method, void *data)
{
  int length;
  SWFOutput header;
  if(movie->nExports > 0)
    SWFMovie_writeExports(movie);

  while(movie->nFrames < movie->totalFrames)
    SWFMovie_nextFrame(movie);

  SWFMovie_addBlock(movie, newSWFEndBlock());
  MingRemoveAllDisplayItems(1);

  length = SWFBlockList_completeBlocks(movie->blockList);

  /* XXX - hack */
  if(movie->displayList->soundStream)
    SWFSound_rewind(movie->displayList->soundStream);

  header = newSizedSWFOutput(20);

  /* figure size, write header */

  SWFOutput_writeRect(header, movie->bounds);
  SWFOutput_writeUInt16(header, (int)floor(movie->rate*256));
  SWFOutput_writeUInt16(header, movie->nFrames);

  SWFOutput_byteAlign(header);
  length += 8 + SWFOutput_length(header);

  method('F', data);
  method('W', data);
  method('S', data);
  method(movie->version, data);
  methodWriteUInt32(length, method, data);
  SWFOutput_writeToMethod(header, method, data);

  destroySWFOutput(header);

  SWFBlockList_writeBlocksToMethod(movie->blockList, method, data);
  return length;
}

                             
  
     
  