/*
    Ming, an SWF output library
    Copyright (C) 2001  Opaque Industries - http://www.opaque.net/

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "blocks/_cl.h"

#include <math.h>

#include "libming.h"
#include "shape_util.h"

extern float Ming_scale;

WM_DLL_A float WM_DLL_B SWFCharacter_getWidth(SWFCharacter character)
{
  return SWFCharacter_getScaledWidth(character) / Ming_scale;
}

WM_DLL_A float WM_DLL_B SWFCharacter_getHeight(SWFCharacter character)
{
  return SWFCharacter_getScaledHeight(character) / Ming_scale;
}


WM_DLL_A void WM_DLL_B SWFShape_setLine(SWFShape shape, unsigned short width,
		      byte r, byte g, byte b, byte a)
{
  SWFShape_setLineStyle(shape, (unsigned short)(Ming_scale*width), (byte)r, (byte)g, (byte)b, (byte)a);
}

WM_DLL_A SWFFill WM_DLL_B SWFShape_addSolidFill(SWFShape shape, byte r, byte g, byte b, byte a)
{
  return newSWFFill(SWFShape_addSolidFillStyle(shape, r, g, b, a));
}

WM_DLL_A SWFFill WM_DLL_B SWFShape_addGradientFill(SWFShape shape, SWFGradient gradient,
				 byte flags)
{
 return newSWFFill(SWFShape_addGradientFillStyle(shape, gradient, flags));
}

WM_DLL_A SWFFill WM_DLL_B SWFShape_addBitmapFill(SWFShape shape, SWFBitmap bitmap, byte flags)
{
  SWFFill fill = newSWFFill(SWFShape_addBitmapFillStyle(shape, bitmap, flags));
  SWFFill_scaleTo(fill, Ming_scale, Ming_scale);
  return fill;
}

WM_DLL_A void WM_DLL_B SWFShape_setLeftFill(SWFShape shape, SWFFill fill)
{
  SWFShape_setLeftFillStyle(shape, fill==NULL ? NULL : fill->fillstyle);
}

WM_DLL_A void WM_DLL_B SWFShape_setRightFill(SWFShape shape, SWFFill fill)
{
  SWFShape_setRightFillStyle(shape, fill==NULL ? NULL : fill->fillstyle);
}

WM_DLL_A void WM_DLL_B SWFShape_drawCharacterBounds(SWFShape shape, SWFCharacter character)
{
  SWFShape_drawLine(shape, SWFCharacter_getWidth(character), 0);
  SWFShape_drawLine(shape, 0, SWFCharacter_getHeight(character));
  SWFShape_drawLine(shape, -SWFCharacter_getWidth(character), 0);
  SWFShape_drawLine(shape, 0, -SWFCharacter_getHeight(character));
}

WM_DLL_A void WM_DLL_B SWFShape_drawCircle(SWFShape shape, int r)
{
  SWFShape_drawArc(shape, r, 0, 360);
}


/* draw an arc of radius r, centered at (x,y), from angle startAngle to angle
   endAngle (measured in degrees clockwise from due north) */

WM_DLL_A void WM_DLL_B SWFShape_drawArc(SWFShape shape, int r, float startAngle, float endAngle)
{
  int i;
  float controlx, controly, anchorx, anchory, x, y;

  /* first determine number of segments, 8 at most */
  int nSegs = 1 + INT_FLOOR(7*(endAngle-startAngle)/360);

  /* subangle is half the angle of each segment */
  float subangle = M_PI*(endAngle-startAngle)/nSegs/360;

  float angle = M_PI*startAngle/180;

  x = (float)rint(r*sin(angle));
  y = (float)-rint(r*cos(angle));

  SWFShape_movePen(shape, x, y);

  for(i=0; i<nSegs; ++i)
  {
    angle += subangle;
    controlx = (float)(r*sin(angle)/cos(subangle));
    controly = (float)(-r*cos(angle)/cos(subangle));
    angle += subangle;
    anchorx = (float)(r*sin(angle));
    anchory =(float)( -r*cos(angle));

    SWFShape_drawCurve(shape,
		       (float)rint(controlx)-x,(float) rint(controly)-y,
		       (float)rint(anchorx-controlx), (float)rint(anchory-controly));
    x = anchorx;
    y = anchory;
  }
}
 

/* x,y relative to shape origin */

WM_DLL_A void WM_DLL_B SWFShape_movePenTo(SWFShape shape, float x, float y)
{
  SWFShape_moveScaledPenTo(shape, (int)rint(x*Ming_scale),
			   (int)rint(y*Ming_scale));
}

WM_DLL_A void WM_DLL_B SWFShape_movePen(SWFShape shape, float dx, float dy)
{
  SWFShape_moveScaledPen(shape, (int)rint(dx*Ming_scale),
			 (int)rint(dy*Ming_scale));
}

WM_DLL_A void WM_DLL_B SWFShape_drawLineTo(SWFShape shape, float x, float y)
{
  SWFShape_drawScaledLineTo(shape, (int)rint(x*Ming_scale),
			    (int)rint(y*Ming_scale));
}

WM_DLL_A void WM_DLL_B SWFShape_drawLine(SWFShape shape, float dx, float dy)
{
  SWFShape_drawScaledLine(shape, (int)rint(dx*Ming_scale),
			  (int)rint(dy*Ming_scale));
}

WM_DLL_A void WM_DLL_B SWFShape_drawCurveTo(SWFShape shape, float controlx, float controly,
			  float anchorx, float anchory)
{
  SWFShape_drawScaledCurveTo(shape,
			     (int)rint(controlx*Ming_scale),
			     (int)rint(controly*Ming_scale),
			     (int)rint(anchorx*Ming_scale),
			     (int)rint(anchory*Ming_scale));
}

WM_DLL_A void WM_DLL_B SWFShape_drawCurve(SWFShape shape,	float controldx, float controldy,
			float anchordx, float anchordy)
{
  SWFShape_drawScaledCurve(shape,
			   (int)rint(controldx*Ming_scale),
			   (int)rint(controldy*Ming_scale),
			   (int)rint(anchordx*Ming_scale),
			   (int)rint(anchordy*Ming_scale));
}

WM_DLL_A void WM_DLL_B SWFShape_drawGlyph(SWFShape shape, SWFFont font, unsigned char c)
{
  SWFShape_drawScaledGlyph(shape, font, c, 1024);
}

WM_DLL_A void WM_DLL_B SWFShape_drawSizedGlyph(SWFShape shape,
			     SWFFont font, unsigned char c, int size)
{
  if(size == 0)
    SWFShape_drawScaledGlyph(shape, font, c, 1024);
  else
    SWFShape_drawScaledGlyph(shape, font, c,(int)( size*Ming_scale));
}

static SWFFill *NewFillsAdded;
static int nNewFillsAdded;
static int nMaxNewFillsAdded;
#define MOVIE_ITEMS_ADDED_INCR 1024
static int ntotal_fills_added;
static int  ntest_compactify_fills_added_at;

static void CompactifyFillsArray(void)
{
 int i=0;
 int ipos=0;
 for(i=0;i<nNewFillsAdded;i++)
  if(NewFillsAdded[i])
  {
   if(ipos==i)
    ipos++;
   else
   {
    SWFFill item=NewFillsAdded[i];
    item->nrecord=i;
    NewFillsAdded[ipos++]=NewFillsAdded[i];
   }
  }
 nNewFillsAdded=ipos;
 ntest_compactify_fills_added_at=ntotal_fills_added+max(1024,nNewFillsAdded/2);
}


WM_DLL_A void WM_DLL_B MingRecordNewFillAdded(SWFFill item)
{
 int i=0;
 if(nNewFillsAdded+1>nMaxNewFillsAdded)
 {
  int new_nMaxNewFillsAdded=nMaxNewFillsAdded+MOVIE_ITEMS_ADDED_INCR;
  SWFFill *NewNewFillsAdded=(SWFFill*)_realloc_((void *)NewFillsAdded,new_nMaxNewFillsAdded*sizeof(SWFFill *));
  if(NewNewFillsAdded)
  {
   NewFillsAdded=NewNewFillsAdded;
   nMaxNewFillsAdded=new_nMaxNewFillsAdded;
  }
  else
  {
   return;
  }
 }
 item->nrecord=nNewFillsAdded;
 NewFillsAdded[nNewFillsAdded++]=item;
 ntotal_fills_added++;
 if(ntotal_fills_added>=ntest_compactify_fills_added_at)
  CompactifyFillsArray();
 return;
}

WM_DLL_A void WM_DLL_B MingUnRecordNewFillRemoved(SWFFill item)
{
 int i=item->nrecord;
 if(i>=0&&i<nNewFillsAdded)
  if(NewFillsAdded[i]==item)
  {
   NewFillsAdded[i]=NULL;
   return;
  }
 return;
}

WM_DLL_A void WM_DLL_B MingRemoveAllFills(void)
{
 int i=0;
 for(i=0;i<nNewFillsAdded;i++)
  if(NewFillsAdded[i])
  {
   destroySWFFill(NewFillsAdded[i]);
   NewFillsAdded[i]=NULL;
  }
  _free_(NewFillsAdded);
  nNewFillsAdded=0;
  nMaxNewFillsAdded=0;
  NewFillsAdded=NULL;
 return;
}

     
  